"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fastify_1 = __importDefault(require("fastify"));
const basic_auth_1 = __importDefault(require("basic-auth"));
const connections_1 = __importDefault(require("./connections"));
const config_1 = __importDefault(require("./config"));
const control_1 = __importDefault(require("./control"));
const discord_1 = __importDefault(require("./discord"));
const messenger_1 = __importDefault(require("./messenger"));
const fastify_static_1 = __importDefault(require("fastify-static"));
const path_1 = __importDefault(require("path"));
const log = logger.withScope('api');
function runServer() {
    const app = fastify_1.default();
    app.register(fastify_static_1.default, {
        root: path_1.default.join(__dirname, '..', '..', 'node_modules'),
        prefix: '/node_modules/',
        decorateReply: false
    });
    app.register(fastify_static_1.default, {
        root: path_1.default.join(__dirname, '..', '..', 'static'),
        prefix: '/static/'
    });
    if (config.api.key) {
        app.addHook('preHandler', (request, reply, next) => {
            const { authorization } = request.headers;
            if (!authorization)
                return reply.code(403).send(new Error('API key missing'));
            if (!authorization.startsWith('Bearer '))
                return reply.code(403).send(new Error('API key incorrect'));
            const key = authorization.split(' ')[1];
            if (key !== config.api.key)
                return reply.code(403).send(new Error('API key incorrect'));
            next();
        });
    }
    else {
        app.addHook('preHandler', (request, reply, next) => {
            const send401 = () => reply
                .code(401)
                .header('WWW-Authenticate', 'Basic realm="Miscord API"')
                .header('Content-Type', 'text/html')
                .send('<h1>Forbidden</h1>');
            if (!request.headers.authorization)
                return send401();
            const creds = basic_auth_1.default.parse(request.headers.authorization);
            if (!creds)
                return send401();
            const { name, pass } = creds;
            log.debug('login', { name, pass });
            if (name === config.api.username && pass === config.api.password)
                return next();
            send401();
        });
    }
    app.get('/', (request, reply) => {
        reply.sendFile('dashboard/index.html');
    });
    app.get('/config-discord/', (request, reply) => {
        reply.sendFile('dashboard/config-discord.html');
    });
    app.get('/config-messenger/', (request, reply) => {
        reply.sendFile('dashboard/config-messenger.html');
    });
    app.get('/config-miscellaneous/', (request, reply) => {
        reply.sendFile('dashboard/config-miscellaneous.html');
    });
    app.get('/connections/', (request, reply) => {
        reply.sendFile('dashboard/connections.html');
    });
    app.register(connections_1.default, { prefix: '/connections' });
    app.register(config_1.default, { prefix: '/config' });
    app.register(control_1.default, { prefix: '/control' });
    app.register(discord_1.default, { prefix: '/discord' });
    app.register(messenger_1.default, { prefix: '/messenger' });
    app.listen(config.api.port || 9448, '0.0.0.0').then(() => log.success(`API is listening on port ${config.api.port || 9448}`));
    return app;
}
exports.default = runServer;
