"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const log = logger.withScope('setup-api');
const fastify_1 = __importDefault(require("fastify"));
const fastify_static_1 = __importDefault(require("fastify-static"));
const events_1 = __importDefault(require("events"));
const discord_js_1 = require("discord.js");
const libfb_1 = require("libfb");
const path_1 = __importDefault(require("path"));
class SetupServer extends events_1.default {
    constructor() {
        super(...arguments);
        this.app = fastify_1.default();
        this.messengerSession = { tokens: null, deviceId: null };
    }
    run() {
        this.app.register(fastify_static_1.default, {
            root: path_1.default.join(__dirname, '..', '..', '..', 'node_modules'),
            prefix: '/node_modules/',
            decorateReply: false
        });
        this.app.register(fastify_static_1.default, {
            root: path_1.default.join(__dirname, '..', '..', '..', 'static', 'setup'),
            prefix: '/static/'
        });
        this.app.get('/', (request, reply) => {
            reply.sendFile('index.html');
        });
        this.app.post('/validate/discord', async (request, reply) => {
            const client = new discord_js_1.Client();
            try {
                await client.login(request.body.token);
                return { valid: true, username: `${client.user.username}#${client.user.discriminator}` };
            }
            catch (err) {
                return { valid: false, error: err.message };
            }
        });
        this.app.post('/validate/messenger', async (request, reply) => {
            const client = new libfb_1.Client();
            try {
                await client.login(request.body.username, request.body.password);
                const userID = client.getSession().tokens.uid;
                const user = await client.getUserInfo(userID);
                this.messengerSession = client.getSession();
                return { valid: true, username: user.name };
            }
            catch (err) {
                return { valid: false, error: err.message };
            }
        });
        this.app.post('/config', async (request, reply) => {
            this.emit('config', request.body);
            return {};
        });
        this.app.listen(9448, '0.0.0.0').then(() => {
            log.success(`API is listening on port 9448`);
            log.info('Go to http://localhost:9448/ for setup');
        });
    }
    stop() {
        return this.app.close();
    }
}
exports.default = SetupServer;
