"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const log = logger.withScope('createMessage:fromMessenger:discord');
const getAttachmentURL_1 = __importDefault(require("../getAttachmentURL"));
const handleEmojis_1 = __importDefault(require("../../discord/handleEmojis"));
const emojiCount_1 = __importDefault(require("../emojiCount"));
const parseMessengerMessage_1 = __importStar(require("./parseMessengerMessage"));
const url_1 = __importDefault(require("url"));
exports.default = async (thread, sender, message) => {
    // set description to message body, set author to message sender
    let authorName;
    ({ authorName, message } = parseMessengerMessage_1.default(thread, sender, message));
    let body = handleEmojis_1.default(message.message);
    if (body.length > 2000)
        body = body.slice(0, 1997) + '...';
    // if there are no attachments, send it already
    const opts = {
        username: authorName.length <= 32 ? emojiCount_1.default(authorName) === 1 ? authorName + '.' : authorName : authorName.substr(0, 29) + '...',
        avatarURL: sender.profilePicLarge,
        files: []
    };
    if ((!message.fileAttachments || !message.fileAttachments.length) &&
        (!message.mediaAttachments || !message.mediaAttachments.length) &&
        !message.stickerId)
        return { body, opts };
    log.trace('attachments to parse', [message.fileAttachments, message.mediaAttachments], Infinity);
    const appendToBody = (str) => { if (!body.includes(str))
        body += '\n' + str; };
    let notSentAttachments = false;
    for (let attach of message.fileAttachments) {
        const url = attach.url || await getAttachmentURL_1.default(message, attach);
        if (!url)
            continue;
        if (attach.size && attach.size > 8 * 1024 * 1024) {
            log.warn('Attachment was not sent due to Discord file size limit', attach);
            notSentAttachments = true;
        }
        opts.files.push({ attachment: url, name: attach.filename });
    }
    if (notSentAttachments) {
        const message = 'Some attachments were not sent due to Discord file size limit';
        appendToBody(message);
        if (config.messenger.attachmentTooLargeError)
            await messenger.client.sendMessage(thread.id, message);
    }
    for (let attach of message.mediaAttachments) {
        if (attach.message)
            appendToBody(attach.message);
        if (attach.description)
            appendToBody(attach.description);
        if (attach.url)
            appendToBody(attach.url);
        if (attach.imageURL) {
            opts.files.push({
                attachment: attach.imageURL,
                name: url_1.default.parse(attach.imageURL).pathname.split('/').slice(-1)[0]
            });
        }
        if (attach.type === 'UnavailableXMA') {
            log.trace('unavailableXMA', attach.attach);
        }
    }
    if (message.stickerId && !parseMessengerMessage_1.thumbs.includes(message.stickerId)) {
        const stickerURL = await messenger.client.getStickerURL(message.stickerId);
        opts.files.push({ attachment: stickerURL, name: message.stickerId + '.png' });
    }
    log.trace('files', opts.files);
    return { body, opts };
};
