"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const log = logger.withScope('createMessage:fromMessenger:messenger');
const getAttachmentURL_1 = __importDefault(require("../getAttachmentURL"));
const downloadFile_1 = __importDefault(require("../downloadFile"));
const parseMessengerMessage_1 = __importStar(require("./parseMessengerMessage"));
exports.default = async (thread, sender, message) => {
    // set description to message body, set author to message sender
    let authorName;
    ({ authorName, message } = parseMessengerMessage_1.default(thread, sender, message));
    let body = config.messenger.format
        .replace('{username}', authorName)
        .replace('{content}', message.message)
        .replace('{message}', message.message)
        .replace('{source}', config.messenger.sourceFormat.messenger.replace('{name}', thread.name))
        .replace('{newline}', '\n');
    if ((!message.fileAttachments || !message.fileAttachments.length) &&
        (!message.mediaAttachments || !message.mediaAttachments.length) &&
        !message.stickerId)
        return { body, attachments: [] };
    const attachments = [];
    for (let attach of message.fileAttachments) {
        const url = attach.url ? attach.url : await getAttachmentURL_1.default(message, attach);
        if (!url)
            continue;
        attachments.push(await downloadFile_1.default(url));
    }
    const appendToBody = (str) => { if (!body.includes(str))
        body += '\n' + str; };
    for (let attach of message.mediaAttachments) {
        if (attach.message)
            appendToBody(attach.message);
        if (attach.description)
            appendToBody(attach.description);
        if (attach.url)
            appendToBody(attach.url);
        if (attach.imageURL)
            attachments.push(await downloadFile_1.default(attach.imageURL));
    }
    if (message.stickerId && !parseMessengerMessage_1.thumbs.includes(message.stickerId)) {
        attachments.push(await messenger.client.getStickerURL(message.stickerId).then(downloadFile_1.default));
    }
    log.trace('attachments', attachments);
    return { body, attachments };
};
