"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const log = logger.withScope('errorHandler');
const is_npm_1 = require("is-npm");
const sentry_1 = require("./sentry");
const utils_1 = require("../utils");
const getErrorDescription_1 = __importDefault(require("./getErrorDescription"));
const FileConfig_1 = require("../config/FileConfig");
const dataPath = process.env.DATA_PATH !== 'undefined' ? process.env.DATA_PATH : undefined;
function tryToGetTheActualError(e) {
    if (e instanceof Error)
        return e;
    if (e.error)
        return tryToGetTheActualError(e.error);
    if (e.err)
        return tryToGetTheActualError(e.err);
    return e;
}
exports.default = async (error) => {
    if (!(error instanceof Error)) {
        const possibleError = tryToGetTheActualError(error);
        if (typeof error === 'string')
            error = new Error(error);
        else if (possibleError)
            error = possibleError;
        else
            error = new Error(error.toString());
    }
    const exitCode = (
    // @ts-ignore
    error.requestArgs ||
        error.message.includes('Incorrect login details were provided') ||
        error.message.includes('EPIPE')) ? 1 : 2;
    log.error('', error);
    sentry_1.sendToSentry(error);
    const desc = getErrorDescription_1.default(error);
    if (desc)
        log.error(desc);
    if (is_npm_1.isNpm) {
        log.warn(`Logs from NPM are unnecessary and don't give much information.
Miscord logs folder:
${dataPath || FileConfig_1.getConfigDir()}/logs`);
    }
    if (global.discord && discord.channels && discord.channels.error) {
        try {
            let errorMessage = utils_1.splitString(`${error.message}\n${error.stack}`, 1000);
            for (let channel of discord.channels.error) {
                for (let part of errorMessage)
                    await channel.send(part, { code: true });
                if (desc)
                    await channel.send(desc);
            }
        }
        catch (err) {
            log.fatal(err);
            sentry_1.sendToSentry(err);
        }
    }
    await sentry_1.closeSentry();
    if (global.discord)
        await discord.client.destroy();
    console.error(`close ${exitCode}`);
};
