"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cleanTemporaryFiles_1 = __importDefault(require("./cleanTemporaryFiles"));
const log = logger.withScope('messenger:listener');
const createMessage_1 = require("../createMessage");
const discord_1 = require("../discord");
const utils_1 = require("../utils");
const _1 = require("./");
exports.default = async (message) => {
    if (!message)
        throw new Error('Message missing!');
    if (config.paused) {
        log.info('Got a Messenger message (paused)');
        return;
    }
    log.trace('message', message);
    log.info('Got a Messenger message');
    if (utils_1.checkMKeep(message.message))
        return log.debug('m!keep received, ignoring.');
    if (utils_1.checkIgnoredSequences(message.message))
        return log.debug('found an ignored sequence, ignoring.');
    // get thread info to know if it's a group conversation
    const thread = await _1.getThread(message.threadId);
    log.debug('Got Messenger thread');
    log.trace('thread', thread, 1);
    // also get sender info
    const sender = await _1.getSender(message.authorId);
    if (!sender)
        return;
    log.debug('Got user info');
    log.trace('sender', sender);
    let connection = await connections.getWithCreateFallback(message.threadId.toString(), thread.cleanName);
    if (!connection)
        return;
    await connection.checkChannelRenames(thread.cleanName);
    {
        const channels = await connection.getWritableChannels();
        const data = await createMessage_1.fromMessenger.toDiscord(thread, sender, message);
        channels.forEach(endpoint => discord_1.sendMessage(endpoint.id, data, thread.image));
    }
    {
        const threads = connection.getOtherWritableThreads(message.threadId);
        const data = await createMessage_1.fromMessenger.toMessenger(thread, sender, message);
        Promise.all(threads.map(thread => _1.sendMessage(thread, data)))
            .then(() => cleanTemporaryFiles_1.default(data));
    }
};
